def returnGoodRegions(x,y,max_sep,min_card):
    #Import(s)
    import numpy as np

    #Action
    dates_list = list(x)
    dates_array = np.array(x)
    differences = list(np.diff(dates_array))
    break_indices = []

    for item in differences:
        if item >= max_sep:
            break_indices.append(differences.index(item)+1)
    break_indices = np.array(break_indices)

    fx = np.split(ary=dates_array,indices_or_sections=break_indices)
    fy = np.split(ary=np.array(y),indices_or_sections=break_indices)
   
    #This section gets rid of intervals that don't meat the min_card limit
    i = 0
    arrays_to_remove = []
    for item in fx:
        if len(item) < min_card:
            arrays_to_remove.append(i)
        i += 1    
    
    for i in sorted(arrays_to_remove, reverse=True): #this is legendary
        del fx[i]
        del fy[i]

    return len(fx), fx, fy



def plotLightCurve(x,y,colors,x_label,y_label,plot_title,line_labels,plot_type,out_type,error_arrays):
    #Import(s)
    import matplotlib.pyplot as plt 
    import numpy as np
    import seaborn as sns
    
    #Action
    sns.set_style('darkgrid')
    plt.rcParams['font.family'] = 'Nimbus Roman'
    plt.title(plot_title)
    plt.xlabel(x_label)
    plt.ylabel(y_label)
    

    item_index = 0
    for item in x:
        if plot_type == 'scatter':
            plt.scatter(item,y[item_index],s=2,c=(colors[item_index]),label=line_labels[item_index])
        elif plot_type == 'plot':
            plt.plot(item,y[item_index],markersize=2,linewidth=1,color=colors[item_index],label=line_labels[item_index])
        elif plot_type == 'Scatter_error':
            plt.error(item,y[item_index],yerr=error_arrays[item_index],marker='o',color=colors[item_index],ms=2,linewidth=0,label=line_labels[item_index])
        elif plot_type == 'plot_error':
            plt.error(item,y[item_index],yerr=error_arrays[item_index],marker='o',color=colors[item_index],ms=2,linewidth=0,label=line_labels[item_index])
        item_index = item_index+1
    
    plt.gca().invert_yaxis()
    plt.legend()
    
    if out_type == 'show':
        plt.show()
        plt.clf()
    else: 
        plt.savefig(out_type)
        plt.clf()



def calculateFluxAsymmetry(x):
    #Description: Returns the flux asymmetry metric M from Bredall et al. 2019
    
    #Import(s)
    import numpy as np
    from scipy import stats

    #Action
    x = list(x)
    
    bottom10 = [] #bottom 10% of magnitude measurements
    top10 = [] #top 10% of magnitude measurements

    for item in x:
        if stats.percentileofscore(x,item) <= 10:
            bottom10.append(item)
        elif stats.percentileofscore(x,item) >= 90:
            top10.append(item)
    
    topandbottom10 = np.array(bottom10+top10)
    m10 = np.mean(topandbottom10)
    x_median = np.median(x) 
    sd = np.std(x)
    M = (m10-x_median)/(sd) #Flux Asymmetry metric, M
    
    return M

def lombScargle(id,x,y,min_period,max_period,out_type):
    #Import(s)
    from astropy.timeseries import LombScargle
    import matplotlib.pyplot as plt
    from astropy.io import ascii
    import numpy as np 

    #Action
    dates = np.array(x)
    mags = np.array(y)
    #Calculate periodogram
    frequencies, powers = LombScargle(dates,mags).autopower(method='fastchi2',minimum_frequency=(1/max_period),maximum_frequency=(1/min_period))
    #Some statistics
    false_alarm_levels = list(LombScargle(dates,mags).false_alarm_level([0.1,0.05,0.01]))
    #Phase the data
    best_frequency = frequencies[np.argmax(powers)]
    T = 1/(float(best_frequency))
    phased_dates = np.mod(dates, T)/T #grabbed this from feets package documentation
    phased_dates_cycle_2 = phased_dates+1

    #Generate plots
    ax1 = plt.subplot(222)
    periods=1/frequencies
    ax1.plot(periods,powers)
    ax1.hlines(y=(false_alarm_levels[0]),xmin=min_period,xmax=max_period,colors=['silver'],linestyles='dashed',label='90% FAP')
    ax1.hlines(y=(false_alarm_levels[1]),xmin=min_period,xmax=max_period,colors=['darkgrey'],linestyles='dashed',label='95% FAP')
    ax1.hlines(y=(false_alarm_levels[2]),xmin=min_period,xmax=max_period,colors=['gray'],linestyles='dashed',label='99% FAP')
    ax1.set_xscale('log')
    ax1.set_xlabel('Period d')
    ax1.set_xscale('log')
    ax1.set_ylabel('Power')
    ax1.set_yticks([0.1,0.3,0.5,0.7,0.9])
    ax1.set_title('Periodogram',fontsize=10)
    ax1.legend(prop={'size':5})

    ax2 = plt.subplot(221)
    ax2.set_xlabel('Phase')
    ax2.set_ylabel('Mag')
    ax2.scatter(phased_dates,mags,s=2)
    ax2.scatter(phased_dates_cycle_2,mags,s=2,c='C0')
    ax2.invert_yaxis()
    ax2.set_title("Folded Light Curve, P: "+str(round((1/best_frequency),3))+' d',fontsize=10)
    ax2.locator_params(axis='y',nbins=5)
    
    ax3 = plt.subplot(212)
    ax3.scatter(dates,mags,s=2)
    ax3.invert_yaxis()
    ax3.set_ylabel('Mag')
    ax3.set_xlabel('HJD')
    ax3.set_title('Light Curve',fontsize=10)
    ax3.locator_params(axis='y',nbins=5)
    

    plt.subplots_adjust(wspace=0.3,hspace=0.45)

    if out_type == 'show':
        plt.show()
        plt.clf()
    else: 
        plt.savefig(out_type,dpi=200)
        plt.clf()
        
def sergisonDistribution(x,percentiles):
    #Description: Returns variability amplitude metric and normalized magnitudes array as in Sergison et al. 2019 
    
    #Import(s)
    import numpy as np
    
    #Action
    mag_list = list(x)
    mean_mag = np.mean(np.array(mag_list))
    lower_percentile,upper_percentile = np.percentile(np.array(mag_list),[percentiles[0], percentiles[1]])
    amp_metric = lower_percentile-upper_percentile
    

    normalized_mags = []

    for item in mag_list:
        normalized_mag = (item-mean_mag)/amp_metric
        normalized_mags.append(normalized_mag)

    return amp_metric, np.array(normalized_mags) # Return results



import setuptools

with open("readme.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="ysospy-noahbraunf",
    version="0.0.2",
    author="Noah Braunfeld",
    author_email="noah@braunfeld.dev",
    description="Useful helper functions for HRL",
    long_description=long_description,
    url="https://github.com/noahbraunf/ysopy",
    packages=setuptools.find_packages(),
    classifiers=["Programming Language :: Python :: 3"],
    python_requires=">=3.6",
)




def calculatePeakLocs(x,w_val):
    #Import(s)
    from scipy.signal import find_peaks
    import numpy as np
    
    #Action
    x = np.array(x)
    fakepeaks, _ = find_peaks(x,width=w_val)
    realpeaks, _ = find_peaks((x*-1),width=w_val)
    outarray = np.concatenate((fakepeaks,realpeaks))
    return(outarray)

def queryCoordSimbad(raw_coord,search_radius):
    #Import(s)
    import numpy as np
    from astropy import coordinates as coord
    from astropy import units as u
    from astroquery.simbad import Simbad
    from astropy.coordinates.sky_coordinate import SkyCoord

    #Action
    c = SkyCoord(raw_coord,unit=(u.hourangle,u.deg))
    c = c.to_string('hmsdms')
    result_table = Simbad.query_region(coord.SkyCoord(c, frame='icrs'),radius=('0d0m'+str(search_radius)+'s'))
    names_col = result_table['MAIN_ID']
    id = str(names_col[0])[1:]
    return id

def returnDistances(x):
    #Import(s)
    import numpy as np
    
    #Action
    input_array = np.array(x)
    out_array = np.diff(input_array)
    return out_array
    
def removeInterval(x,y,interval):
    #Import(s)
    import numpy as np 

    #Action
    x = list(x)

    new_x = []
    indices = []
    new_ys = []
    lower_bound = float(interval.split(':')[0])
    upper_bound = float(interval.split(':')[1])    

    for item in x:
        if item > upper_bound or item < lower_bound:
            new_x.append(item)
            indices.append(x.index(item))

    for item in y:
        item = list(item)
        new_y = []
        for elem in indices:
            new_y.append(item[elem])
        new_ys.append(np.array(new_y))

    return np.array(new_x),new_ys

def sortData(x,y):
    #Import(s)
    import numpy as np
    
    #Action
    
    unsorted_dates = list(x)
    
    sorted_dates = list(np.sort(unsorted_dates))
    sorted_arrays = []
    for item in y:
        unsorted_list = list(item)
        sorted_list = []
        for elem in sorted_dates:
            #newIndex = sorted_dates.index(elem)
            oldIndex = unsorted_dates.index(elem)
            sorted_list.append(unsorted_list[oldIndex])
        sorted_array = np.array(sorted_list)
        sorted_arrays.append(sorted_array)

    sorted_dates = np.array(sorted_dates)
    
    return sorted_dates, sorted_arrays

def returnSI(data):
    n = 0
    tempTup = []
    data = list(data)
    for elem in data:
        tempTup.append((elem, n))
        n += 1
    tempTup.sort()
    new_indices = []
    for elem in tempTup:
        new_indices.append(elem[1])
    return new_indices

def sortFI(data, indices): #indices must be less than or equal to data
    data=list(data)
    indices=list(indices)
    rearranged = []
    for elem in indices:
        rearranged.append(data[elem])
    return rearranged   
